-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftVisibility")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Critter"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Critter"] = "동물",
} end)

-----------------------------------------------------------------------------

local AloftVisibility = Aloft:NewModule("Visibility", "AceEvent-2.0")
AloftVisibility.db = Aloft:AcquireDBNamespace("visibility")
Aloft:RegisterDefaults("visibility", "profile", {
	showHostilePlayers = true,
	showHostilePets = true,
	showHostileNPCs = true,

	friendlyPlayers = "ALL",
	showPet = true,
	showGroupPets = true,
	showFriendlyPets = true,
	showFriendlyNPCs = true,

	showNeutralUnits = true,
	showCritters = true,
})

-----------------------------------------------------------------------------

function AloftVisibility:Update()
	self:RegisterEvents()

	-- Toggle the visibility to get previously hidden stuff to show
	Aloft:RedisplayNameplates()
	Aloft:RedisplayFriendlyNameplates()
end

function AloftVisibility:RegisterEvents()
	self:UnregisterAllEvents()

	if not self.db.profile.showHostilePlayers then
		self:RegisterEvent("Aloft:OnClassDataChanged", "OnClassDataChanged")
	end
	
	if self.db.profile.showNeutralUnits and not self.db.profile.showCritters then
		self:RegisterEvent("Aloft:OnCreatureTypeDataChanged", "OnCreatureTypeDataChanged")
	end
   
	if not self.db.profile.showHostilePets or
	   not self.db.profile.showFriendlyPets then
		self:RegisterEvent("Aloft:OnIsPetDataChanged", "OnNameplateShow")
	end

	if not self.db.profile.showCritters or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showHostilePets or
	   not self.db.profile.showHostileNPCs or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   not self.db.profile.showFriendlyPets or
	   not self.db.profile.showGroupPets or
	   not self.db.profile.showPet or
	   self.db.profile.friendlyPlayers ~= "ALL" then
	   
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	end
end

-----------------------------------------------------------------------------

function AloftVisibility:OnEnable()	
	self:RegisterEvents()
end

function AloftVisibility:OnDisable()	
	self:UnregisterAllEvents()

	if not self.db.profile.showCritters or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showHostilePets or
	   not self.db.profile.showHostileNPCs or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   not self.db.profile.showFriendlyPets  or
	   not self.db.profile.showGroupPets or
	   not self.db.profile.showPet or
	   self.db.profile.friendlyPlayers ~= "ALL" then
		Aloft:RedisplayNameplates()
		Aloft:RedisplayFriendlyNameplates()
	end
end

local dataRequiredList = { }
function AloftVisibility:RequiresData()
	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if not self.db.profile.showHostilePlayers then
		table.insert(dataRequiredList, "class")
	end

	if self.db.profile.showHostilePets ~= self.db.profile.showHostileNPCs or 
	   self.db.profile.showFriendlyPets ~= self.db.profile.showFriendlyNPCs then
		table.insert(dataRequiredList, "isPet")
	end

	if self.db.profile.friendlyPlayers == "GUILDONLY" then
		table.insert(dataRequiredList, "isGuildMember")
	end

	if self.db.profile.showNeutralUnits and not self.db.profile.showCritters then
		table.insert(dataRequiredList, "creatureType")
	end

	if Aloft:IsDataAvailable("unitid") then
		if self.db.profile.friendlyPlayers == "GROUPONLY" or
	   	   self.db.profile.showGroupPets ~= self.db.profile.showFriendlyPets then
			table.insert(dataRequiredList, "unitid")
		end
	end

	return unpack(dataRequiredList)
end

-----------------------------------------------------------------------------

local friendlyPlayerMethods =
{
	["ALL"] = function() end,
	["GUILDONLY"] = function(frame, aloftData) if not aloftData.isGuildMember then frame:Hide() end end,
	["GROUPONLY"] = function(frame, aloftData) if not aloftData.unitid then frame:Hide() end end,
	["NONE"] = function(frame) frame:Hide() end,
}

local showTypeMethods =
{
	["friendlyPlayer"] = function(frame, aloftData) 
							local friendlyPlayerMethod = friendlyPlayerMethods[AloftVisibility.db.profile.friendlyPlayers]
							if friendlyPlayerMethod then
								friendlyPlayerMethod(frame, aloftData)
							end
						 end,
	["friendlyNPC"] = function(frame, aloftData) 
						if UnitName("pet") == aloftData.name then
							if not AloftVisibility.db.profile.showPet then
								frame:Hide()
							end
						elseif aloftData.unitid then
							-- Group pet
							if not AloftVisibility.db.profile.showGroupPets then
								frame:Hide()
							end
						elseif aloftData.isPet then
							if not AloftVisibility.db.profile.showFriendlyPets then
								frame:Hide()
							end
						else
							if not AloftVisibility.db.profile.showFriendlyNPCs then 
								frame:Hide() 
							end
						end
					  end,
	["neutral"] = function(frame, aloftData) 
					if not AloftVisibility.db.profile.showNeutralUnits or (not AloftVisibility.db.profile.showCritters and aloftData.creatureType == L["Critter"]) then 
						frame:Hide()
					end
				  end,
	["hostile"] = function(frame, aloftData)
					-- Is it a player?
					if aloftData.class then 
						if not AloftVisibility.db.profile.showHostilePlayers then
							frame:Hide()
						end
					elseif aloftData.isPet then
						if not AloftVisibility.db.profile.showHostilePets then
							frame:Hide()
						end
					else
						if not AloftVisibility.db.profile.showHostileNPCs then
							frame:Hide()
						end
					end
				  end,
	["unknown"] = function() end,
}

function AloftVisibility:OnNameplateShow(aloftData)
	showTypeMethods[aloftData.type](aloftData.nameplateFrame, aloftData)
end

function AloftVisibility:OnClassDataChanged(aloftData)
	-- If it was changed, then it must have some value now
	if aloftData.type == "hostile" then
		aloftData.nameplateFrame:Hide()
	end
end

function AloftVisibility:OnCreatureTypeDataChanged(aloftData)
	if aloftData.type == "neutral" and aloftData.creatureType == L["Critter"] then
		aloftData.nameplateFrame:Hide()
	end
end

-----------------------------------------------------------------------------
